// DCC Decoder for DS-DCC decode
// By yaasan
// Based on Nicolas's sketch http://blog.nicolas.cx
// Inspired by Geoff Bunza and his 17 Function DCC Decoder & updated library
//
//
// Debug //Serial output available on the //Serial port at baud 115200, aka Tools -> //Serial Monitor
//
//150520 fujigaya2 with like VVVF sound ?
//150523 fujigaya2 with CV2,5 variable
//150604 fujigaya2 with chord ?
//150615 fujigaya2 swiching sound CV47 
//150625 fujigaya2 with melody,separate VVVF array(header)

#include <NmraDcc.h>
#include <avr/eeprom.h>	 //required by notifyCVRead() function if enabled below
#include <avr/pgmspace.h>
#include <TimerOne.h>

//Melody用
#define PWMDAC_OUTPUT_PIN   3
#include <PWMDAC_Synth.h>

#include "kasoku.h" //VVVF配列
//#include "hana.h" //Melody
#include "hana_k1.h" //Melody
#include "hana2_k1.h" //Melody
#include "tanabatasama.h" //Melody
//#include "railroad_crossing.h" //Melody
#include "door_open_close.h" //Melody
//#include "GK5-1_2.h"

//各種設定、宣言

#define DECODER_ADDRESS 3
#define DCC_ACK_PIN 9	//if defined enables the ACK pin functionality. Comment out to disable.

#define MOTOR_PWM_A 9
#define MOTOR_PWM_B 10

#define MOTOR_LIM_MAX 255

#define CV_VSTART		2
#define CV_ACCRATIO		3
#define CV_DECCRATIO	4
#define CV_VHIGH	5//fujigaya2
#define CV_SOUNDSWITCH 47//fujigaya2
#define CV_SOUNDSTATE 48//fujigaya2


//使用クラスの宣言
NmraDcc	 Dcc;
DCC_MSG	 Packet;

//Task Schedule
unsigned long gPreviousL5 = 0;

int kasoku[15][8];
int uplimit;

int pwm_duty = 32;
double pwm_freq = 440;
#define HOKAN_BUF_NUM  60//速度履歴バッファ大きさ
int pwm_hokan[HOKAN_BUF_NUM];//速度履歴
uint8_t preDir = 0;//一個前にDCCからの方向情報
int current_buf_pos = 0;//Bufferの書き換え位置

int current_period1 =  (double)1000000 / pwm_freq;//TimerOneに渡すPeriod1
int current_period2 = current_period1;//TimerOneに渡すPeriod2
int pwm_state = 0;//Period1,2切替
int current_duty_a = 0;//TimerOneに渡すduty
int current_duty_b = 0;//TimerOneに渡すduty

//モータ制御関連の変数
uint32_t gPwmLPF_buf = 0;
uint32_t gPwmLPF2_buf = 0;
uint8_t gPwmDir = 128;
uint16_t gSpeedCmd = 0;
uint16_t gPrevSpeed = 0;

uint8_t gCV1_SAddr = 3;
uint8_t gCVx_LAddr = 3;
uint8_t gCV2_Vstart = 32;
uint8_t gCV3_AccRatio = 32;
uint8_t gCV4_DecRatio = 32;
uint8_t gCV5_Vhigh = 32;
uint8_t gCV47_SoundSwitch = 0;
uint8_t gCV48_SoundState = 1;

//Internal variables and other.
#if defined(DCC_ACK_PIN)
const int DccAckPin = DCC_ACK_PIN ;
#endif

/* Function assigned pins */
const int FunctionPin0 = 4;
const int FunctionPin1 = 5;
const int FunctionPin2 = 6;
const int FunctionPin3 = 7;
const int FunctionPin4 = 8;

struct CVPair{
  uint16_t	CV;
  uint8_t	Value;
};
CVPair FactoryDefaultCVs [] = {
  {CV_MULTIFUNCTION_PRIMARY_ADDRESS, DECODER_ADDRESS},
  {CV_ACCESSORY_DECODER_ADDRESS_MSB, 0},		//The LSB is set CV 1 in the libraries .h file, which is the regular address location, so by setting the MSB to 0 we tell the library to use the same address as the primary address. 0 DECODER_ADDRESS
  {CV_MULTIFUNCTION_EXTENDED_ADDRESS_MSB, 0},	 //XX in the XXYY address
  {CV_MULTIFUNCTION_EXTENDED_ADDRESS_LSB, 0},	 //YY in the XXYY address
  {CV_29_CONFIG, 128 },	 //Make sure this is 0 or else it will be random based on what is in the eeprom which could caue headaches
  {CV_VSTART, 16},
  {CV_ACCRATIO, 64},
  {CV_DECCRATIO, 64},
  {CV_VHIGH, 64},
  {CV_SOUNDSWITCH, 0},
  {CV_SOUNDSTATE, 1}

};

//Melody用
int v = 0;
int c= 0;
bool tenmetu = false;
int notes_size = 0;
unsigned long time = 0;
//buffer
byte notes1_value = 0;
byte notes2_value = 0;
byte notes3_value = 0;
byte notes4_value = 0;
byte notes5_value = 0;
byte pitch_value =  0;
//オクターブオフセット
byte note_offset;
//melodyのリピート回数カウント
int melody_repeat;
//door_open_close　flag
bool door_open_close_melody_flag = false;
int door_func_old_value = 0;
//melody_select
int melody_no = 0;

//DCC PWM_Synthの切替フラグ
enum loop_state {l_pre_dcc,l_dcc,l_pre_melody,l_melody};
loop_state l_state = l_dcc;

void(* resetFunc) (void) = 0;  //declare reset function at address 0


uint8_t FactoryDefaultCVIndex = sizeof(FactoryDefaultCVs) / sizeof(CVPair);

uint16_t limitSpeed(uint16_t inSpeed);


void notifyCVResetFactoryDefault()
{
	//When anything is writen to CV8 reset to defaults. 

	resetCVToDefault();	 
	Serial.println(F("Resetting..."));
	delay(1000);  //typical CV programming sends the same command multiple times - specially since we dont ACK. so ignore them by delaying

	resetFunc();
};


void resetCVToDefault()
{
	//CVをデフォルトにリセット
	Serial.println(F("CVs being reset to factory defaults"));
	
	for (int j=0; j < FactoryDefaultCVIndex; j++ ){
		Dcc.setCV( FactoryDefaultCVs[j].CV, FactoryDefaultCVs[j].Value);
	}
};



extern void	   notifyCVChange( uint16_t CV, uint8_t Value){
   //CVが変更されたときのメッセージ
   Serial.print(F("CV ")); 
   Serial.print(CV); 
   Serial.print(F(" Changed to ")); 
   Serial.println(Value, DEC);
};

void notifyCVAck(void)
{
  Serial.println(F("notifyCVAck"));
  
  analogWrite(MOTOR_PWM_B, 0);
  analogWrite(MOTOR_PWM_A, 64);
  
  delay( 6 );  
  
  analogWrite(MOTOR_PWM_A, 0);
}

//Melody用
void note_OnOff(byte &note_value,byte offset,int count,int delay_c)
{
  //note_value:音程
  //offset 音程のオフセット、＋１２で1オクターブ上になる
  //count 音を出すタイミングと消すタイミング
  //delay_c 音を出す／消すタイミングのdelay
  if(note_value != 0)
  {
    if(count == (0+delay_c))
    {
      //点ける
      PWM_SYNTH.noteOn(1,note_value + offset,120);
    }
    else if(count == (1+delay_c))
    {
      //消す
      PWM_SYNTH.noteOff(1,note_value + offset,120);
    }
  }
}

void setup()
{
	uint8_t cv_value;
	
	//シリアル通信開始
	Serial.begin(115200);

	//D9,D10 PWM キャリア周期:31kHz
	//TCCR1B &= B11111000;
	//TCCR1B |= B00000001;
	
	//PWMを10bit化
	//TCCR1A |= B00000011;

	
	//PWM出力ピン D9,D10を出力にセット
	pinMode(MOTOR_PWM_A, OUTPUT);
	pinMode(MOTOR_PWM_B, OUTPUT);

        //Timer1使用
        Timer1.initialize();
        //初期設定
        Timer1.pwm(MOTOR_PWM_A,0,0);
        Timer1.pwm(MOTOR_PWM_B,0,0);
        Timer1.attachInterrupt(int1,current_period1);//interruptの設定
        for(int i= 0 ;i < HOKAN_BUF_NUM;i++)
        {
          pwm_hokan[i] =  0;
        }
        
	//ファンクションの割り当てピン初期化
	pinMode(FunctionPin0, OUTPUT);
	digitalWrite(FunctionPin0, 0);
	
	pinMode(FunctionPin1, OUTPUT);
	digitalWrite(FunctionPin1, 0);
	
	pinMode(FunctionPin2, OUTPUT);
	digitalWrite(FunctionPin2, 0);	
	
	pinMode(FunctionPin3, OUTPUT);
	digitalWrite(FunctionPin3, 0);
	
	pinMode(FunctionPin4, OUTPUT);
	digitalWrite(FunctionPin4, 0);
	
	
	//DCCの応答用負荷ピン
	
	#if defined(DCCACKPIN)
	//Setup ACK Pin
	//pinMode(DccAckPin,OUTPUT);
	//digitalWrite(DccAckPin, 0);
	#endif
  
 

  #if !defined(DECODER_DONT_DEFAULT_CV_ON_POWERUP)
	if ( Dcc.getCV(CV_MULTIFUNCTION_PRIMARY_ADDRESS) == 0xFF ){	 //if eeprom has 0xFF then assume it needs to be programmed
	  Serial.println(F("CV Defaulting due to blank eeprom"));
	  notifyCVResetFactoryDefault();
	  
   } else{
	 Serial.println(F("CV Not Defaulting"));
   }
  #else
	 Serial.println(F("CV Defaulting Always On Powerup"));
	 notifyCVResetFactoryDefault();
  #endif 
  
   
   
  // Setup which External Interrupt, the Pin it's associated with that we're using, disable pullup.
  Dcc.pin(0, 2, 0);
  
  // Call the main DCC Init function to enable the DCC Receiver
  Dcc.init( MAN_ID_DIY, 100,   FLAGS_MY_ADDRESS_ONLY , 0 ); 

  //Reset task
  gPreviousL5 = millis();
  
  //Init CVs
  gCV1_SAddr = Dcc.getCV( CV_MULTIFUNCTION_PRIMARY_ADDRESS ) ;
  gCV2_Vstart = Dcc.getCV( CV_VSTART ) ;
  gCV3_AccRatio = Dcc.getCV( CV_ACCRATIO ) ;
  gCV4_DecRatio = Dcc.getCV( CV_DECCRATIO ) ;
  gCV5_Vhigh = Dcc.getCV( CV_VHIGH ) ;
  gCV47_SoundSwitch = Dcc.getCV(CV_SOUNDSWITCH);
  gCV48_SoundState = Dcc.getCV(CV_SOUNDSTATE);
  gCVx_LAddr = (Dcc.getCV( CV_MULTIFUNCTION_EXTENDED_ADDRESS_MSB ) << 8) + Dcc.getCV( CV_MULTIFUNCTION_EXTENDED_ADDRESS_LSB );
  
  Serial.print(F("CV1(ShortAddr): "));
  Serial.println(gCV1_SAddr);
  Serial.print(F("CV17/18(LongAddr): "));
  Serial.println(gCVx_LAddr);
  Serial.print(F("CV2(Vstart): "));
  Serial.println(gCV2_Vstart);
  Serial.print(F("CV3(AccRatio): "));
  Serial.println(gCV3_AccRatio);
  Serial.print(F("CV2(DecRatio): "));
  Serial.println(gCV4_DecRatio);
  Serial.print(F("CV5(Vhigh): "));
  Serial.println(gCV5_Vhigh);
  Serial.print(F("CV47(SoundSwitch): "));
  Serial.println(gCV47_SoundSwitch);
  Serial.print(F("CV48(SoundState): "));
  Serial.println(gCV48_SoundState);

  //安全策
  if(gCV47_SoundSwitch > 8)
  {
    gCV47_SoundSwitch = 8;
  }
  int kasoku_offset =  (int)pgm_read_word_near(&progmem_ref_kasoku[gCV47_SoundSwitch]);
  uplimit = (int)pgm_read_word_near(&progmem_ref_kasoku[gCV47_SoundSwitch+1]) - kasoku_offset;
  for(int i = 0;i < uplimit;i++)
  {
    for (int j = 0;j < 8;j++)
    {
      kasoku[i][j] =  (int)pgm_read_word_near(&progmem_kasoku [i + kasoku_offset][j]); 
      Serial.print(kasoku[i][j],DEC);
      Serial.print(",");
    }
    Serial.println("");
  }        

  Serial.println(F("Ready"));
  
  //Melody用
    PWM_SYNTH.setup();
  // すべてのMIDIチャンネルに波形をセット
  //PWM_SYNTH.setWave( PWMDACSynth::sineWavetable ); 
  //PWM_SYNTH.setWave( PWMDACSynth::squareWavetable );
  //PWM_SYNTH.setWave( PWMDACSynth::triangleWavetable );
  PWM_SYNTH.setWave( PWMDACSynth::sawtoothWavetable );
  //PWM_SYNTH.setWave( PWMDACSynth::shepardToneSineWavetable );
  //PWM_SYNTH.setWave( PWMDACSynth::maxVolumeSineWavetable );
  
  //オクターブオフセット +12で1オクターブ高くなる
  note_offset = 24;
  //Repeat回数
  melody_repeat = 0;
  
  //notes_size = sizeof( notes1);
  time = micros();

  //initial state
  l_state = l_pre_dcc;
}

//temp_debug用
//int debug_count = 0;

void select_melody()
{
  //copy note from ROM. because ROM is too slow. 
  //melody or door_open_close
  if(door_open_close_melody_flag == true)
  {
    notes1_value = pgm_read_byte_near(&notes_door1[v]);
    notes2_value = pgm_read_byte_near(&notes_door2[v]);
    notes3_value = pgm_read_byte_near(&notes_door3[v]);
    notes4_value = pgm_read_byte_near(&notes_door4[v]);
    notes5_value = pgm_read_byte_near(&notes_door5[v]);
    pitch_value = pgm_read_byte_near(&pitch_door[v]);
    notes_size = sizeof( notes_door1);
  }
  else
  {
    switch(melody_no)
    {
      case 0:
      {
        //hana1
        notes1_value = pgm_read_byte_near(&notes_hana1[v]);
        notes2_value = pgm_read_byte_near(&notes_hana2[v]);
        notes3_value = pgm_read_byte_near(&notes_hana3[v]);
        notes4_value = pgm_read_byte_near(&notes_hana4[v]);
        notes5_value = pgm_read_byte_near(&notes_hana5[v]);
        pitch_value = pgm_read_byte_near(&pitch_hana[v]);
        notes_size = sizeof( notes_hana1);
        break;
      }
      case 1:
      {
        //hana1
        notes1_value = pgm_read_byte_near(&notes_hana2_1[v]);
        notes2_value = pgm_read_byte_near(&notes_hana2_2[v]);
        notes3_value = pgm_read_byte_near(&notes_hana2_3[v]);
        notes4_value = pgm_read_byte_near(&notes_hana2_4[v]);
        notes5_value = pgm_read_byte_near(&notes_hana2_5[v]);
        pitch_value = pgm_read_byte_near(&pitch_hana2[v]);
        notes_size = sizeof( notes_hana2_1);
        break;
      }
      case 2:
      {
        //tanabatasama  
        notes1_value = pgm_read_byte_near(&notes_tanabatasama1[v]);
        notes2_value = pgm_read_byte_near(&notes_tanabatasama2[v]);
        notes3_value = pgm_read_byte_near(&notes_tanabatasama3[v]);
        notes4_value = pgm_read_byte_near(&notes_tanabatasama4[v]);
        notes5_value = pgm_read_byte_near(&notes_tanabatasama5[v]);
        pitch_value = pgm_read_byte_near(&pitch_tanabatasama[v]);
        notes_size = sizeof( notes_tanabatasama1);
        break;
      }
      default:
      {
        //hana
        notes1_value = pgm_read_byte_near(&notes_hana1[v]);
        notes2_value = pgm_read_byte_near(&notes_hana2[v]);
        notes3_value = pgm_read_byte_near(&notes_hana3[v]);
        notes4_value = pgm_read_byte_near(&notes_hana4[v]);
        notes5_value = pgm_read_byte_near(&notes_hana5[v]);
        pitch_value = pgm_read_byte_near(&pitch_hana[v]);
        notes_size = sizeof( notes_hana1);
        break;
      }
    }
  }    
}


void loop()
{
  //Stateで分ける
  switch (l_state)
  {
    case l_pre_dcc:
    {
      //prepare dcc mode
      //disable PWM_Synth
      TIMSK2 &= ~_BV(TOIE2);//Disable Interrupt
      //enable Timer1
      Timer1.pwm(MOTOR_PWM_A,0,0);
      Timer1.pwm(MOTOR_PWM_B,0,0);
      Timer1.attachInterrupt(int1,current_period1);//interruptの設定
      //next state is "l_dcc".
      l_state = l_dcc;
      break;
    }
    
    case l_dcc:
    {
      	// You MUST call the NmraDcc.process() method frequwently from the Arduino loop() function for correct library operation
      	Dcc.process();
      	if( (millis() - gPreviousL5) >= 33)
      	{
      		//Motor drive control
      		Motor_Control();
      		
      		//Reset task
      		gPreviousL5 = millis();
      		
      	}
//      debug_count ++;
//      if(debug_count == 5)
//        l_state = l_pre_melody;
      break;
    }
    case l_pre_melody:
    {
      //prepare melody mode
      //disable timer1
      Timer1.detachInterrupt();
      //enable PWM_Synth
      TIMSK2 |= _BV(TOIE2);//Enable Interrupt
      //net state is "l_melody"
      melody_repeat = 0;
      l_state = l_melody;
      break;
    }
    case l_melody:
    {
      //play Melody
      if((time + 1500) < micros())
      {
        time = micros();
        //max 5 chord
        if(c == 0)
        {
          select_melody();
        }
        if(pitch_value == 255)
        { 
          //ぽろろろーん
          note_OnOff(notes1_value,note_offset,c,4 * 50);
          note_OnOff(notes2_value,note_offset,c,3 * 50);
          note_OnOff(notes3_value,note_offset,c,2 * 50);
          note_OnOff(notes4_value,note_offset,c,1 * 50);
          note_OnOff(notes5_value,note_offset,c,0 * 50);
        }
        else
        {
          //normal
          note_OnOff(notes1_value,note_offset,c,0);
          note_OnOff(notes2_value,note_offset,c,0);
          note_OnOff(notes3_value,note_offset,c,0);
          note_OnOff(notes4_value,note_offset,c,0);
          note_OnOff(notes5_value,note_offset,c,0);
        }          
        c ++;
        if(c > (int)pitch_value)
        {
          c = 0;
          tenmetu = !tenmetu;
          v ++;
          if(v == notes_size)
          {
            v = 0;
            melody_repeat ++;
            //door_open_close is only 1 time;
            door_open_close_melody_flag = false;
          }
        }
        PWM_SYNTH.updateEnvelopeStatus(); 
        //return dccmode if 1 repeat。
        if(melody_repeat == 1)
        {
          l_state = l_pre_dcc;
        }
      }
      break;
    }
    default:
    {
      //it don't come this state.
      break;
    }
  }
  
}

/* Motor control Task (30Hz) */
void Motor_Control()
{
  	uint16_t aPWMRef = 0;
        //適当な平均化ルーチン（2秒分,30Hz時）
        //for(int i =0;i<HOKAN_BUF_NUM - 1;i++)
        //{
        //  pwm_hokan[i] = pwm_hokan[i+1];
        //}
        pwm_hokan[current_buf_pos] = gSpeedCmd * 4;
        current_buf_pos ++;
        if(current_buf_pos >= HOKAN_BUF_NUM)
        {
          current_buf_pos = 0;
        }   
        //方向が違うときは０リセット
        if(preDir != gPwmDir)
        {
          for(int i=0;i<HOKAN_BUF_NUM;i++)
          {
            pwm_hokan[i] = 0;
          }
        }
        preDir = gPwmDir;
        //HOKAN_BUF_NUM回の平均化
        double temp_pwm = 0;
        for(int i= 0;i<HOKAN_BUF_NUM;i++)
        {
          temp_pwm += (double)pwm_hokan[i];
        } 
        aPWMRef = (uint16_t)(temp_pwm / HOKAN_BUF_NUM);  
        if(aPWMRef != 0)
        {
          for(int i = 0;i < uplimit;i++)
          {
            if ( (kasoku[i][0] <= aPWMRef ) and ( aPWMRef <= kasoku [i][1]))
            {
              //音色側
              //周波数１
              pwm_freq = (double)kasoku[i][2]
                       + (double)(kasoku[i][3] - kasoku[i][2]) / (double)(kasoku [i][1] - kasoku [i][0])
                       * (double)(aPWMRef - kasoku [i][0]);
              current_period1 = (double)1000000 / pwm_freq;//interruptで渡す
              //Serial.print(pwm_freq,DEC);
              //Serial.print(F(" Hz,"));
              //周波数２
              pwm_freq = (double)kasoku[i][4]
                       + (double)(kasoku[i][5] - kasoku[i][4]) / (double)(kasoku [i][1] - kasoku [i][0])
                       * (double)(aPWMRef - kasoku [i][0]);
              current_period2 = (double)1000000 / pwm_freq;//interruptで渡す
              //Serial.print(pwm_freq,DEC);
              //Serial.print(F(" Hz,"));
              //電力側
              pwm_duty = (double)kasoku[i][6]
                + (double)(kasoku[i][7] - kasoku[i][6]) / (double)(kasoku [i][1] - kasoku [i][0])
                * (double)(aPWMRef - kasoku [i][0]);
              //CV2,CV5で修正
              pwm_duty = (double)gCV2_Vstart * 4 + (double)pwm_duty * (double)(gCV5_Vhigh - gCV2_Vstart) / (double)255;
              if (pwm_duty >1023)
              {
                pwm_duty = 1023;
              }
              //Serial.print(F("Duty:"));
              //Serial.println(pwm_duty,DEC);
              if(gPwmDir > 0)
              {
                //interruptで渡す
                current_duty_b = 0;
                current_duty_a = pwm_duty;
              }
              else
              {
                //interruptで渡す
                current_duty_a = 0;
                current_duty_b = pwm_duty;
              }
            }
          }
        }
        else
        {
          //interruptで渡す
          current_duty_b = 0;
          current_duty_a = 0;
        }          
}



uint16_t limitSpeed(uint16_t inSpeed)
{
	uint16_t aSpeed_ref = 0;
	
	if( inSpeed >= MOTOR_LIM_MAX)
	{
		aSpeed_ref = MOTOR_LIM_MAX;
	}
	else if( inSpeed <= 0)
	{
		aSpeed_ref = 0;
	}
/*	else if( (inSpeed > 0) && (inSpeed <= gCV2_Vstart))
	{
		aSpeed_ref = gCV2_Vstart;
	}*/
	else
	{
		aSpeed_ref = inSpeed;
	}
	
	return aSpeed_ref;
	
}


//DCC速度信号の受信によるイベント
extern void notifyDccSpeed( uint16_t Addr, uint8_t Speed, uint8_t ForwardDir, uint8_t MaxSpeed )
{
	
	
	uint16_t aSpeedRef = 0;
	
	//速度値の正規化(255を100%とする処理)
	if( Speed >= 1)
	{
		aSpeedRef = ((Speed - 1) * 255) / MaxSpeed;
	}
	else
	{
		aSpeedRef = 0;
	}
	
	//リミッタ
	if(aSpeedRef > 255)
	{
		aSpeedRef = 255;
	}
	
	gSpeedCmd = aSpeedRef;
	gPwmDir = ForwardDir;
	
	
	// デバッグメッセージ
/*	//Serial.print("Speed - ADR: ");
	//Serial.print(Addr);
	//Serial.print(", SPD: ");
	//Serial.print(Speed);
	//Serial.print(", DIR: ");
	//Serial.print(ForwardDir);
	//Serial.print(", MAX: ");
	//Serial.println(MaxSpeed);
*/
}

//ファンクション信号受信のイベント
extern void notifyDccFunc( uint16_t Addr, FN_GROUP FuncGrp, uint8_t FuncState)
{
	switch(FuncGrp)
	{
		case FN_0_4:	//Function Group 1 F0 F4 F3 F2 F1
			exec_function( 0, FunctionPin0, (FuncState & FN_BIT_00)>>4 );
			exec_function( 1, FunctionPin1, (FuncState & FN_BIT_01));
			exec_function( 2, FunctionPin2, (FuncState & FN_BIT_02)>>1);
                        exec_function( 3, FunctionPin3, (FuncState & FN_BIT_03)>>2 );
			exec_function( 4, FunctionPin4, (FuncState & FN_BIT_04)>>3 );
			break;

		case FN_5_8:	//Function Group 1 S FFFF == 1 F8 F7 F6 F5	&  == 0	 F12 F11 F10 F9 F8
                        if((FuncState & FN_BIT_05) != door_func_old_value)
			{
                          //if func6 changed,door_open_close_melody_flag ->true
                          //play Melody if loco is stopped. 
                          if((current_duty_a == 0) && (current_duty_b == 0))
                          { 
                            l_state = l_pre_melody;
                            door_open_close_melody_flag = true;
                          }
                        }
                        door_func_old_value = (FuncState & FN_BIT_05);
                        if(((FuncState & FN_BIT_06)>>1 ) != 0)
                        {
                          //play Melody if loco is stopped. 
                          if((current_duty_a == 0) && (current_duty_b == 0))
                          { 
                            l_state = l_pre_melody;
                            melody_no = 0;
                          }
                        }
                        if(((FuncState & FN_BIT_07)>>2) != 0)
                        {
                          //play Melody if loco is stopped. 
                          if((current_duty_a == 0) && (current_duty_b == 0))
                          { 
                            l_state = l_pre_melody;
                            melody_no = 1;
                          }
                        }
                        if(((FuncState & FN_BIT_08)>>3) != 0)
                        {
                          //play Melody if loco is stopped. 
                          if((current_duty_a == 0) && (current_duty_b == 0))
                          { 
                            l_state = l_pre_melody;
                            melody_no = 2;
                          }
                        }
                        //exec_function( 5, FunctionPin5, (FuncState & FN_BIT_05));
			//exec_function( 6, FunctionPin6, (FuncState & FN_BIT_06)>>1 );
			//exec_function( 7, FunctionPin7, (FuncState & FN_BIT_07)>>2 );
			//exec_function( 8, FunctionPin8, (FuncState & FN_BIT_08)>>3 );
			break;

		case FN_9_12:
			//exec_function( 9, FunctionPin9,	(FuncState & FN_BIT_09));
			//exec_function( 10, FunctionPin10, (FuncState & FN_BIT_10)>>1 );
			//exec_function( 11, FunctionPin11, (FuncState & FN_BIT_11)>>2 );
			//exec_function( 12, FunctionPin12, (FuncState & FN_BIT_12)>>3 );
			break;

		case FN_13_20:	 //Function Group 2 FuncState == F20-F13 Function Control
			//exec_function( 13, FunctionPin13, (FuncState & FN_BIT_13));
			//exec_function( 14, FunctionPin14, (FuncState & FN_BIT_14)>>1 );
			//exec_function( 15, FunctionPin15, (FuncState & FN_BIT_15)>>2 );
			//exec_function( 16, FunctionPin16, (FuncState & FN_BIT_16)>>3 );
			break;

		case FN_21_28:
			break;
			
	}
}

void exec_function (int function, int pin, int FuncState)
{
	digitalWrite (pin, FuncState);
}

void int1()
{
  //和音生成用interrupt
  switch(pwm_state)
  {
    case 0:
    {
      Timer1.setPeriod(current_period1);
      break;
    }
    default:
    {
      Timer1.setPeriod(current_period2);
      break;
    }
  }
  //pwm_stateのトグル
  if(pwm_state == gCV48_SoundState)
  {
    pwm_state = 0;
  }
  else
  {
    pwm_state ++;
  }
  
  //Duty
  Timer1.setPwmDuty(MOTOR_PWM_A,current_duty_a);
  Timer1.setPwmDuty(MOTOR_PWM_B,current_duty_b);
}
